## Set file paths
inFilePath <- file.path("C:", "Users/Dganghel/Desktop", "data_1871_1979_replication.csv")
outFilePath <- file.path("C:", "Users/Dganghel/Desktop", "output.txt")

## Import data
data <- read.csv(inFilePath, sep=";")

## Compute long run exponential growth trend
g <- (data[nrow(data), "P"] / data[1, "P"])^(1/(nrow(data)-1))-1

## Detrend prices
data$"detrended(P)" <- data[,"P"]
for (i in 1:nrow(data)) { data[i, "detrended(P)"] <- data[i, "detrended(P)"]/(1+g)^(i-nrow(data))}

## Detrend dividends
data$"detrended(D)" <- data[,"D"]
for (i in 1:nrow(data)) { data[i, "detrended(D)"] <- data[i, "detrended(D)"]/(1+g)^(i-nrow(data))}

## Compute constant discount rate
r <- mean(data[,"detrended(D)"]) / mean(data[,"detrended(P)"])

## Set terminal price 
tp <- mean(data[,"detrended(P)"])

## Compute fundamental values
data$"p*" <- data[,"detrended(P)"]
data[nrow(data),"p*"] <- tp
for (i in (nrow(data)-1):1) { data[i,"p*"] = (data[i+1,"p*"] + data[i,"detrended(D)"]) / (1 + r) }

##################################
######### OPTIONAL CODE ##########
### Compute lower_ fundamental value estimate by changing the terminal condition
#data$"lower_" <- data[,"detrended(P)"]
#data[nrow(data),"lower_"] <- tp - sd(data[,"detrended(P)"])
#for (i in (nrow(data)-1):1) { data[i,"lower_"] = (data[i+1,"lower_"] + data[i,"detrended(D)"]) / (1 + r) }
#
### Compute upper_ fundamental value estimate by changing the terminal condition
#data$"upper_" <- data[,"detrended(P)"]
#data[nrow(data),"upper_"] <- tp + sd(data[,"detrended(P)"])
#for (i in (nrow(data)-1):1) { data[i,"upper_"] = (data[i+1,"upper_"] + data[i,"detrended(D)"]) / (1 + r) }
##################################
##################################

# Export results
write.table(data,outFilePath,sep="\t",row.names=FALSE)
